import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Metadata, Http, I18n } from '@zeedhi/core';
import { DialogService, LoadingService } from '@zeedhi/common';
import { PmLibAlertService, PmLibDialogService } from '@zeedhi/pm-lib';

export class ControleLogSistemaController {
    public grid = Metadata.getInstance<TekGrid>('controleLogSistemaGrid');

    public get checkedAsVisualizedCallActionDisabled() {
        const callGridInstance = Metadata.getInstance<TekGrid>('controleLogSistemaGrid');
        return callGridInstance.selectedRows.length === 0;
    }

    public dialogEventsAction: any = {
        click: async ({ component }: any) => {
            if (component.name === 'ENABLE' || component.name === 'DISABLE') {
                try {
                    LoadingService.show();
                    this.grid.selectedRows.forEach((element) => {
                        element.CONDITION = component.name;
                    });
                    await Http.post('/updateTrigger', this.grid.selectedRows);
                    LoadingService.hide();
                    DialogService.hide();
                    PmLibAlertService.showSuccess(I18n.translate('Registro(s) salvo(s) com sucesso.'));
                    this.grid.selectedRows = [];
                    await this.grid.reload();
                } catch (e) {
                    LoadingService.hide();
                    PmLibDialogService.showErrorBackend(e);
                }
            } else {
                DialogService.hide();
            }
        },
    };

    public clickControleLogSistemaAction() {
        DialogService.show({
            name: 'action-dialog',
            type: 'warning',
            title: I18n.translate('Selecione a opção desejada'),
            text: I18n.translate('Você deseja habilitar ou desabilitar as tabelas selecionadas?'),
            buttons: [
              { name: 'ENABLE', component: 'ZdButton', label: I18n.translate('Habilitar'), events: this.dialogEventsAction },
              { name: 'DISABLE', component: 'ZdButton', label: I18n.translate('Desabilitar'), events: this.dialogEventsAction },
              { name: 'CANCEL', component: 'ZdButton', label: I18n.translate('Cancelar'), events: this.dialogEventsAction, outline: true },
            ],
          });
    }
}
