import { Http, Loader, Metadata, Singleton, I18n } from '@zeedhi/core';
import { OperadorController } from '@/controllers/Operador/OperadorController';
import { OperadorService } from '@/services/OperadorService';
import { Form, IGridEvent, LoadingService, Modal, ModalService } from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Date as DateInput } from '@zeedhi/common/types/components/zd-date/date';
import { PmLibAlertService, PmLibDialogService, PmLibHttp } from '@zeedhi/pm-lib';

@Singleton
export class TabUnidadesRelacionadasController {
	public operadorControler: OperadorController;

	public operadorService: OperadorService;

	constructor() {
		this.operadorControler = Loader.getInstance('OperadorController');
		this.operadorService = new OperadorService();
	}

	/** Grid Component */

	public getRelateUnidadeByOperadorGrid(): TekGrid {
		return Metadata.getInstance<TekGrid>('relateUnidadesByOperadorGrid');
	}

	public getUnidadesRelacionadasGrid(): TekGrid {
		return Metadata.getInstance<TekGrid>('unidadesRelacionadasGrid');
	}

	public async onMountedUnidadesRelacionadasGrid() {
		this.relateUnidadesByOperadorModal = await ModalService.createFromJson(this.relateUnidadesByOperadorModalPath, true);
	}

	public async addClickUnidadesRelacionadasGrid() {
		await this.relateUnidadesByOperadorModal.show();
		const grid = this.getRelateUnidadeByOperadorGrid();
		grid.datasource.addFilter('CDOPERADOR', this.operadorControler.currentOperador.CDOPERADOR);
	}

	public async beforeSaveUnidadesRelacionadasGrid({ event }: IGridEvent) {
		event?.preventDefault();
		this.updateFilioper();
	}

	private async updateFilioper() {
		const grid = this.getUnidadesRelacionadasGrid();
		const modifiedItems = grid.getEditedRows();
		const dataset = modifiedItems.map((row) => ({
			CDOPERADOR: this.operadorControler.currentOperador.CDOPERADOR,
			CDFILIAL: row.CDFILIAL,
			IDATIVO: row.IDATIVO,
			DTVALIDADE: row.DTVALIDADE,
		}));

		try {
			LoadingService.show();
			await Http.put('/filioper', dataset);
			LoadingService.hide();
			PmLibAlertService.showSuccess(I18n.translate('Registro(s) salvo(s) com sucesso.'));
			await grid.reload();
			await grid.cancelEditedRows();
		} catch (e) {
			LoadingService.hide();
			PmLibDialogService.showErrorBackend(e);
		}
	}

	public async beforeDeleteUnidadesRelacionadasGrid({ event }: IGridEvent) {
		event?.preventDefault();
		const confirm = await PmLibDialogService.confirmDialog(I18n.translate('Deseja realmente excluir o(s) registro(s)?'));
		if (confirm) {
			this.deleteFilioper();
		}
	}

	private async deleteFilioper() {
		const grid = this.getUnidadesRelacionadasGrid();
		await PmLibHttp.$delete('/filioper', { data: grid.selectedRows });
		PmLibAlertService.showSuccess(I18n.translate('Registro(s) excluído(s) com sucesso.'));
		await grid.reload();
	}

	public onDestroyedGridUnidadesRelacionadasGrid() {
		this.relateUnidadesByOperadorModal.destroy();
	}

	/** Form Modal Component */
	public relateUnidadesByOperadorModal!: Modal;

	public relateUnidadesByOperadorModalPath: string = 'components/operador/detail/tabs/unidadesRelacionadas/modais/relateUnidadesByOperador';

	public async closeRelateUnidadeByOperadorModal() {
		this.relateUnidadesByOperadorModal.hide();
		this.getUnidadesRelacionadasGrid().reload();
	}

	public get validateSaveButton(): boolean {
		const grid = this.getRelateUnidadeByOperadorGrid();
		return !grid.selectedRows.length;
	}

	public async createFilioper() {
		const grid = this.getRelateUnidadeByOperadorGrid();
		const form = Metadata.getInstance<Form>('relateUnidadesByOperadorForm');
		const dtValidateField = form.getChildInstance<DateInput>('DTVALIDATE');
		const { selectedRows } = grid;
		const dataset = selectedRows.map((row) => ({
			DTVALIDADE: dtValidateField.value,
			CDOPERADOR: this.operadorControler.currentOperador.CDOPERADOR,
			CDFILIAL: row.CDFILIAL,
		}));

		try {
			LoadingService.show();
			await Http.post('/filioper', dataset);
			LoadingService.hide();
			PmLibAlertService.showSuccess(I18n.translate('Unidade(s) relacionada(s) com sucesso.'));
			dtValidateField.setDateValue('');
			grid.selectedRows = [];
			await grid.reload();
		} catch (e) {
			LoadingService.hide();
			PmLibDialogService.showErrorBackend(e);
		}
	}
}
