import { Http, Loader, Metadata, Singleton, I18n } from '@zeedhi/core';
import { Form, IGridEvent, LoadingService, Modal, ModalService } from '@zeedhi/common';
import { TekGrid, TekRestDatasource } from '@zeedhi/teknisa-components-common';
import { PmLibAlertService, PmLibDialogService } from '@zeedhi/pm-lib';
import { PerfilDeAcessoController } from '@/controllers/PerfilDeAcesso/PerfilDeAcessoController';
import { Date as DateInput } from '@zeedhi/common/types/components/zd-date/date';

@Singleton
export class TabPerfilDeAcessoOperadorController {
	public perfilDeAcessoControler: PerfilDeAcessoController;

	constructor() {
		this.perfilDeAcessoControler = Loader.getInstance('PerfilDeAcessoController');
	}

	public async onActiveTab() {
		const operatorGrid = Metadata.getInstance<TekGrid>('perfilDeAcessoOperadorGrid');
		await operatorGrid.datasource.addFilter('NRPERFILACESSO', this.perfilDeAcessoControler.currentPerfilDeAcesso.NRPERFILACESSO);
	}

	private getPerfilDeAcessoOperadorGrid(): TekGrid {
		return Metadata.getInstance<TekGrid>('perfilDeAcessoOperadorGrid');
	}

	public async addClickPerfilDeAcessoOperadorGrid() {
		await this.createAddOperadorByPerfilModal();
		await this.addOperadorByPerfilModal.show();
		const addOperadorModalGrid = this.getAddOperadorByPerfilGrid();
		const nrperfilacesso = this.perfilDeAcessoControler.currentPerfilDeAcesso.NRPERFILACESSO;
		await addOperadorModalGrid.datasource.addFilter('NRPERFILACESSO', nrperfilacesso);
	}

	public async beforeDeletePerfilDeAcessoOperadorGrid({ event }: IGridEvent) {
		event?.preventDefault();
		const confirm = await PmLibDialogService.confirmDialog(I18n.translate('Deseja realmente excluir o(s) registro(s)?'));
		if (confirm) {
			this.deleteOperador();
		}
	}

	public async deleteOperador() {
		try {
			const grid = this.getPerfilDeAcessoOperadorGrid();
			LoadingService.show();
			await Http.delete('/perfacessparc', { data: grid.selectedRows });
			LoadingService.hide();
			PmLibAlertService.showSuccess(I18n.translate('Registro(s) excluído(s) com sucesso.'));
			await grid.reload();
		} catch (e) {
			LoadingService.hide();
			PmLibDialogService.showErrorBackend(e);
		}
	}

	/** Add Form Modal Operadores Component */

	public addOperadorByPerfilModal!: Modal;

	public addOperadorByPerfilModalPath: string = 'components/perfil-de-acesso/detail/tabs/tab-items/perfilDeAcessoOperador/modais/addOperadorByPerfil';

	public async createAddOperadorByPerfilModal() {
		this.addOperadorByPerfilModal = await ModalService.createFromJson(this.addOperadorByPerfilModalPath, true);
	}

	private getAddOperadorByPerfilGrid(): TekGrid {
		return Metadata.getInstance<TekGrid>('addOperadorByPerfilGrid');
	}

	public async closeAddOperadorByPerfilModal() {
		this.addOperadorByPerfilModal.hide();
		this.addOperadorByPerfilModal?.destroy();
		await this.getPerfilDeAcessoOperadorGrid().reload();
	}

	public get validateSaveButtonAddOperadorByPerfilModal(): boolean {
		const addOperadorModalGrid = this.getAddOperadorByPerfilGrid();
		return !addOperadorModalGrid.selectedRows.length;
	}

	public async saveAddOperadorByPerfilModal() {
		const addOperadorModalGrid = this.getAddOperadorByPerfilGrid();
		const addOperadorModalForm = Metadata.getInstance<Form>('addOperadorByPerfilForm');
		const dtValidateField = addOperadorModalForm.getChildInstance<DateInput>('DTVALIDATE');
		const { selectedRows } = addOperadorModalGrid;
		selectedRows.forEach((row) => {
			row.DTVALIDADE = dtValidateField.value;
			row.NRPERFILACESSO = this.perfilDeAcessoControler.getCurrentNrperfilacesso();
		});
		try {
			LoadingService.show();
			await Http.post('/perfacessparc', selectedRows);
			LoadingService.hide();
			PmLibAlertService.showSuccess(I18n.translate('Operador(es) adicionado(s) com sucesso.'));
			dtValidateField.setDateValue('');
			addOperadorModalGrid.selectedRows = [];
			await addOperadorModalGrid.reload();
		} catch (e) {
			LoadingService.hide();
			PmLibDialogService.showErrorBackend(e);
		}
	}

	public async onMountedPerfilDeAcessoOperadorGrid() {
		const grid = Metadata.getInstance<TekGrid>('perfilDeAcessoOperadorGrid');
		if (grid.datasource instanceof TekRestDatasource) {
			grid.datasource.dynamicFilter = {
				IDATIVO: [{
					operation: 'EQUALS',
					relation: 'AND',
					value: 'S',
				}],
			};
		}
	}
}
